@@ -394,8 +394,8 @@ func TestParseTimestamp(t *testing.T) {
 		format    string
 		timestamp interface{}
 		location  string
+		separator []string
 		expected  time.Time
-		err       bool
 	}{
 		{
 			name:      "parse layout string in utc",
@@ -404,13 +404,6 @@ func TestParseTimestamp(t *testing.T) {
 			location:  "UTC",
 			expected:  rfc3339("2019-02-20T21:50:34Z"),
 		},
-		{
-			name:      "parse layout string with invalid timezone",
-			format:    "2006-01-02 15:04:05",
-			timestamp: "2019-02-20 21:50:34",
-			location:  "InvalidTimeZone",
-			err:       true,
-		},
 		{
 			name:      "layout regression 6386",
 			format:    "02.01.2006 15:04:05",
@@ -447,6 +440,48 @@ func TestParseTimestamp(t *testing.T) {
 			timestamp: "1568338208.00000050042",
 			expected:  rfc3339("2019-09-13T01:30:08.000000500Z"),
 		},
+		{
+			name:      "unix seconds with thousand separator only (dot)",
+			format:    "unix",
+			timestamp: "1.568.338.208",
+			separator: []string{","},
+			expected:  rfc3339("2019-09-13T01:30:08Z"),
+		},
+		{
+			name:      "unix seconds with thousand separator only (comma)",
+			format:    "unix",
+			timestamp: "1,568,338,208",
+			separator: []string{"."},
+			expected:  rfc3339("2019-09-13T01:30:08Z"),
+		},
+		{
+			name:      "unix seconds with thousand separator only (space)",
+			format:    "unix",
+			timestamp: "1 568 338 208",
+			separator: []string{"."},
+			expected:  rfc3339("2019-09-13T01:30:08Z"),
+		},
+		{
+			name:      "unix seconds with thousand separator only (underscore)",
+			format:    "unix",
+			timestamp: "1_568_338_208",
+			separator: []string{"."},
+			expected:  rfc3339("2019-09-13T01:30:08Z"),
+		},
+		{
+			name:      "unix seconds with thousand and decimal separator (US)",
+			format:    "unix",
+			timestamp: "1,568,338,208.500",
+			separator: []string{"."},
+			expected:  rfc3339("2019-09-13T01:30:08.500Z"),
+		},
+		{
+			name:      "unix seconds with thousand and decimal separator (EU)",
+			format:    "unix",
+			timestamp: "1.568.338.208,500",
+			separator: []string{","},
+			expected:  rfc3339("2019-09-13T01:30:08.500Z"),
+		},
 		{
 			name:      "unix seconds integer",
 			format:    "unix",
@@ -459,6 +494,12 @@ func TestParseTimestamp(t *testing.T) {
 			timestamp: float64(1568338208.500),
 			expected:  rfc3339("2019-09-13T01:30:08.500Z"),
 		},
+		{
+			name:      "unix seconds float exponential",
+			format:    "unix",
+			timestamp: float64(1.5683382085e+9),
+			expected:  rfc3339("2019-09-13T01:30:08.500Z"),
+		},
 		{
 			name:      "unix milliseconds",
 			format:    "unix_ms",
@@ -466,10 +507,10 @@ func TestParseTimestamp(t *testing.T) {
 			expected:  rfc3339("2019-09-13T01:30:08.500Z"),
 		},
 		{
-			name:      "unix milliseconds with fractional is ignored",
+			name:      "unix milliseconds with fractional",
 			format:    "unix_ms",
 			timestamp: "1568338208500.42",
-			expected:  rfc3339("2019-09-13T01:30:08.500Z"),
+			expected:  rfc3339("2019-09-13T01:30:08.50042Z"),
 		},
 		{
 			name:      "unix microseconds",
@@ -483,6 +524,12 @@ func TestParseTimestamp(t *testing.T) {
 			timestamp: "1568338208000000500",
 			expected:  rfc3339("2019-09-13T01:30:08.000000500Z"),
 		},
+		{
+			name:      "unix nanoseconds exponential",
+			format:    "unix_ns",
+			timestamp: "1.5683382080000005e+18",
+			expected:  rfc3339("2019-09-13T01:30:08.000000500Z"),
+		},
 		{
 			name:      "rfc339 test",
 			format:    "RFC3339",
@@ -591,13 +638,75 @@ func TestParseTimestamp(t *testing.T) {
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			tm, err := ParseTimestamp(tt.format, tt.timestamp, tt.location)
-			if tt.err {
-				require.Error(t, err)
-			} else {
-				require.NoError(t, err)
-				require.Equal(t, tt.expected, tm)
-			}
+			tm, err := ParseTimestamp(tt.format, tt.timestamp, tt.location, tt.separator...)
+			require.NoError(t, err)
+			require.Equal(t, tt.expected, tm)
+		})
+	}
+}
+
+func TestParseTimestampInvalid(t *testing.T) {
+	tests := []struct {
+		name      string
+		format    string
+		timestamp interface{}
+		location  string
+		expected  string
+	}{
+		{
+			name:      "too few digits",
+			format:    "2006-01-02 15:04:05",
+			timestamp: "2019-02-20 21:50",
+			expected:  "cannot parse \"\" as \":\"",
+		},
+		{
+			name:      "invalid timezone",
+			format:    "2006-01-02 15:04:05",
+			timestamp: "2019-02-20 21:50:34",
+			location:  "InvalidTimeZone",
+			expected:  "unknown time zone InvalidTimeZone",
+		},
+		{
+			name:      "invalid layout",
+			format:    "rfc3399",
+			timestamp: "09.07.2019 00:11:00",
+			expected:  "cannot parse \"09.07.2019 00:11:00\" as \"rfc\"",
+		},
+		{
+			name:      "layout not matching time",
+			format:    "rfc3339",
+			timestamp: "09.07.2019 00:11:00",
+			expected:  "cannot parse \"7.2019 00:11:00\" as \"2006\"",
+		},
+		{
+			name:      "unix wrong type",
+			format:    "unix",
+			timestamp: true,
+			expected:  "unsupported type",
+		},
+		{
+			name:      "unix multiple separators (dot)",
+			format:    "unix",
+			timestamp: "1568338.208.500",
+			expected:  "invalid number",
+		},
+		{
+			name:      "unix multiple separators (comma)",
+			format:    "unix",
+			timestamp: "1568338,208,500",
+			expected:  "invalid number",
+		},
+		{
+			name:      "unix multiple separators (mixed)",
+			format:    "unix",
+			timestamp: "1,568,338,208.500",
+			expected:  "invalid number",
+		},
+	}
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			_, err := ParseTimestamp(tt.format, tt.timestamp, tt.location)
+			require.ErrorContains(t, err, tt.expected)
 		})
 	}
 }
