@@ -1,11 +1,12 @@
+//go:generate ../../../tools/readme_config_includer/generator
 package clickhouse
 
 import (
 	"bytes"
+	_ "embed"
 	"encoding/json"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"net"
 	"net/http"
 	"net/url"
@@ -20,74 +21,11 @@ import (
 	"github.com/influxdata/telegraf/plugins/inputs"
 )
 
+//go:embed sample.conf
+var sampleConfig string
+
 var defaultTimeout = 5 * time.Second
 
-var sampleConfig = `
-  ## Username for authorization on ClickHouse server
-  ## example: username = "default"
-  username = "default"
-
-  ## Password for authorization on ClickHouse server
-  ## example: password = "super_secret"
-
-  ## HTTP(s) timeout while getting metrics values
-  ## The timeout includes connection time, any redirects, and reading the response body.
-  ##   example: timeout = 1s
-  # timeout = 5s
-
-  ## List of servers for metrics scraping
-  ## metrics scrape via HTTP(s) clickhouse interface
-  ## https://clickhouse.tech/docs/en/interfaces/http/
-  ##    example: servers = ["http://127.0.0.1:8123","https://custom-server.mdb.yandexcloud.net"]
-  servers         = ["http://127.0.0.1:8123"]
-
-  ## If "auto_discovery"" is "true" plugin tries to connect to all servers available in the cluster
-  ## with using same "user:password" described in "user" and "password" parameters
-  ## and get this server hostname list from "system.clusters" table
-  ## see
-  ## - https://clickhouse.tech/docs/en/operations/system_tables/#system-clusters
-  ## - https://clickhouse.tech/docs/en/operations/server_settings/settings/#server_settings_remote_servers
-  ## - https://clickhouse.tech/docs/en/operations/table_engines/distributed/
-  ## - https://clickhouse.tech/docs/en/operations/table_engines/replication/#creating-replicated-tables
-  ##    example: auto_discovery = false
-  # auto_discovery = true
-
-  ## Filter cluster names in "system.clusters" when "auto_discovery" is "true"
-  ## when this filter present then "WHERE cluster IN (...)" filter will apply
-  ## please use only full cluster names here, regexp and glob filters is not allowed
-  ## for "/etc/clickhouse-server/config.d/remote.xml"
-  ## <yandex>
-  ##  <remote_servers>
-  ##    <my-own-cluster>
-  ##        <shard>
-  ##          <replica><host>clickhouse-ru-1.local</host><port>9000</port></replica>
-  ##          <replica><host>clickhouse-ru-2.local</host><port>9000</port></replica>
-  ##        </shard>
-  ##        <shard>
-  ##          <replica><host>clickhouse-eu-1.local</host><port>9000</port></replica>
-  ##          <replica><host>clickhouse-eu-2.local</host><port>9000</port></replica>
-  ##        </shard>
-  ##    </my-onw-cluster>
-  ##  </remote_servers>
-  ##
-  ## </yandex>
-  ##
-  ## example: cluster_include = ["my-own-cluster"]
-  # cluster_include = []
-
-  ## Filter cluster names in "system.clusters" when "auto_discovery" is "true"
-  ## when this filter present then "WHERE cluster NOT IN (...)" filter will apply
-  ##    example: cluster_exclude = ["my-internal-not-discovered-cluster"]
-  # cluster_exclude = []
-
-  ## Optional TLS Config
-  # tls_ca = "/etc/telegraf/ca.pem"
-  # tls_cert = "/etc/telegraf/cert.pem"
-  # tls_key = "/etc/telegraf/key.pem"
-  ## Use TLS but skip chain & host verification
-  # insecure_skip_verify = false
-`
-
 type connect struct {
 	Cluster  string `json:"cluster"`
 	ShardNum int    `json:"shard_num"`
@@ -120,16 +58,10 @@ type ClickHouse struct {
 	tls.ClientConfig
 }
 
-// SampleConfig returns the sample config
 func (*ClickHouse) SampleConfig() string {
 	return sampleConfig
 }
 
-// Description return plugin description
-func (*ClickHouse) Description() string {
-	return "Read metrics from one or many ClickHouse servers"
-}
-
 // Start ClickHouse input service
 func (ch *ClickHouse) Start(telegraf.Accumulator) error {
 	timeout := defaultTimeout
@@ -590,7 +522,7 @@ func (ch *ClickHouse) execQuery(address *url.URL, query string, i interface{}) e
 	}
 	defer func() { _ = resp.Body.Close() }()
 	if resp.StatusCode >= 300 {
-		body, _ := ioutil.ReadAll(io.LimitReader(resp.Body, 200))
+		body, _ := io.ReadAll(io.LimitReader(resp.Body, 200))
 		return &clickhouseError{
 			StatusCode: resp.StatusCode,
 			body:       body,
@@ -606,7 +538,7 @@ func (ch *ClickHouse) execQuery(address *url.URL, query string, i interface{}) e
 		return err
 	}
 
-	if _, err := io.Copy(ioutil.Discard, resp.Body); err != nil {
+	if _, err := io.Copy(io.Discard, resp.Body); err != nil {
 		return err
 	}
 	return nil
